C**********************************************************************
C**********************************************************************
C** 
C**     File: flf.for
C**
C**     Project: jitax77
C**
C**     Description: Routines for writing and reading Photonics Science
C**                  FLF files.
C**                  
C**********************************************************************
C**
C** Copyright (c) 2003 Scott A. Belmonte
C**
C** E-mail: scott.belmonte@ntlworld.com
C**
C** This file is part of jitax77.
C**
C** jitax77 is free software; you can redistribute it and/or modify
C** it under the terms of the GNU General Public License as published
C** by the Free Software Foundation; either version 2 of the License,
C** or (at your option) any later version.
C**
C** jitax77 is distributed in the hope that it will be useful,
C** but WITHOUT ANY WARRANTY; without even the implied warranty of
C** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C** GNU General Public License for more details.
C**
C** You should have received a copy of the GNU General Public License
C** along with jitax77; if not, write to the Free Software
C** Foundation, Inc., 59 Temple Place, Suite 330, Boston,
C** MA  02111-1307  USA.
C**
C**********************************************************************
C**********************************************************************
C
C**********************************************************************
C     Routine: WRITE_FLF_FILE
C
C     Description:
C        Take the data in the DATA array whose dimensions are
C        WIDTH by HEIGHT and writes them to an FLF format file
C        called NAME
C
C        Returns the number of saturated pixels in the frame.
C        Saturated pixels are pixels whose values are greater
C        than 65535. Saturated pixels are set to 65535.
C**********************************************************************
      INTEGER FUNCTION WRITE_FLF_FILE(NAME, DATA, WIDTH, HEIGHT)
      IMPLICIT NONE
      INCLUDE 'cmdln.inc'
      INCLUDE 'flf.inc'
C
C     Parameters
C
      CHARACTER*(*) NAME        ! FLF file name
      INTEGER       DATA(*)     ! The data to write
      INTEGER       WIDTH       ! Image width
      INTEGER       HEIGHT      ! Image height
C
C     Functions
C
      INTEGER*2 WRITE_FLF_SHORT
      LOGICAL   BIGEND_CPU
      INTEGER STRLEN
C
C     Variables
C
      INTEGER      I            ! Loop counter
      INTEGER      NSATPIX      ! Number of saturated pixels
      LOGICAL      SWAP         ! If .TRUE. then swap bytes
C
C
      IF (FLFSIZE .LT. (WIDTH*HEIGHT+16)) GOTO 901
      OPEN(UNIT=12, FILE=NAME, STATUS='UNKNOWN', FORM='UNFORMATTED',
     $     ACCESS='DIRECT', RECL=FLFSIZE*2, ERR=902)
C
C     Initialise header bytes
C
      SWAP = (BIGEND_CPU() .XOR. BIGEND_FLF)
      BUF(1) = WRITE_FLF_SHORT(22950, SWAP)
      BUF(2) = WRITE_FLF_SHORT(27285, SWAP)
      BUF(3) = WRITE_FLF_SHORT(0, SWAP)
      BUF(4) = WRITE_FLF_SHORT(WIDTH, SWAP)
      BUF(5) = WRITE_FLF_SHORT(0, SWAP)
      BUF(6) = WRITE_FLF_SHORT(HEIGHT, SWAP)
      BUF(7) = WRITE_FLF_SHORT(0, SWAP)
      BUF(8) = WRITE_FLF_SHORT(16, SWAP)
      BUF(9) = WRITE_FLF_SHORT(0, SWAP)
      BUF(10) = WRITE_FLF_SHORT(0, SWAP)
      BUF(11) = WRITE_FLF_SHORT(0, SWAP)
      BUF(12) = WRITE_FLF_SHORT(1, SWAP)
      BUF(13) = WRITE_FLF_SHORT(0, SWAP)
      BUF(14) = WRITE_FLF_SHORT(0, SWAP)
      BUF(15) = WRITE_FLF_SHORT(0, SWAP)
      BUF(16) = WRITE_FLF_SHORT(0, SWAP)
C
C     Write converted data to buffer
C
      NSATPIX = 0
      DO I = 1, (WIDTH*HEIGHT)
         IF (DATA(I) .GT. 65535) NSATPIX = NSATPIX + 1
         BUF(I+16) = WRITE_FLF_SHORT(DATA(I), SWAP)
      ENDDO
C
      WRITE(UNIT=12, REC=1) BUF
      CLOSE(UNIT=12)
      WRITE_FLF_FILE = NSATPIX
      RETURN
C
C     Error trap
C
 901  WRITE(*,*) '** Error: Data size', (WIDTH*HEIGHT+16),
     $     ' greater than ',FLFSIZE
      WRITE(*,*) '** Increase FLFSIZE and recompile.'
      STOP
 902  WRITE(*,*) '** Error opening FLF for output: ', NAME
      STOP
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_FLF_FILE
C
C     Description:
C        Reads a 16-bit FLF file into the array DATA
C
C        Returns 0 if successful, 1 otherwise.
C**********************************************************************
      INTEGER FUNCTION READ_FLF_FILE(NAME, DATA, WIDTH, HEIGHT)
      IMPLICIT NONE
      INCLUDE 'flf.inc'
C
C     Parameters
C
      CHARACTER*88 NAME
      INTEGER      DATA(*)
      INTEGER      WIDTH
      INTEGER      HEIGHT
C
C     Functions
C
      INTEGER READ_FLF_SHORT
C
C     Variables
C
      INTEGER    I              ! Loop counter
      INTEGER    FLFW
      INTEGER    FLFH
      LOGICAL    SWAP
C
      IF (FLFSIZE .LT. (WIDTH*HEIGHT+16)) GOTO 911
      OPEN(UNIT=13, FILE=NAME, STATUS='OLD', FORM='UNFORMATTED',
     $     ACCESS='DIRECT', RECL=FLFSIZE*2, ERR=912)
      READ(UNIT=13, REC=1) BUF
      CLOSE(UNIT=13)
C
      IF (BUF(1) .EQ. 22950) THEN
         SWAP = .FALSE.
      ELSE
         SWAP = .TRUE.
      ENDIF
      FLFW = READ_FLF_SHORT(BUF(4), SWAP)
      FLFH = READ_FLF_SHORT(BUF(6), SWAP)
C
      DO I = 1, (FLFW*FLFH)
          DATA(I) = READ_FLF_SHORT(BUF(I+16), SWAP)
      ENDDO
C
      READ_FLF_FILE = 0
      RETURN
C
C
C     Error trap
C
 911  WRITE(*,*) '** Error: Data size', (WIDTH*HEIGHT+16),
     $     ' greater than ',FLFSIZE
      WRITE(*,*) '** Increase FLFSIZE and recompile.'
      STOP
 912  READ_FLF_FILE = 1
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: WRITE_FLF_SHORT
C
C     Description:
C        Converts an INTEGER into INTEGER*2
C**********************************************************************
      INTEGER*2 FUNCTION WRITE_FLF_SHORT(DATA, SWAP)
      IMPLICIT NONE
C
C     Parameters
C
      INTEGER DATA
      LOGICAL SWAP
C
C     Variables
C
      BYTE      TMPB
      BYTE      B(2)
      INTEGER*2 I2
      EQUIVALENCE (B, I2)
C
      IF (DATA .LT. 0) THEN
         I2 = 0
      ELSE IF (DATA .GT. 65535) THEN
         I2 = 65535
      ELSE
         I2 = DATA
      ENDIF
C
      IF (SWAP) THEN
         TMPB = B(1)
         B(1) = B(2)
         B(2) = TMPB
      ENDIF
C
      WRITE_FLF_SHORT = I2
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_FLF_SHORT
C
C     Description:
C        Converts an INTEGER*2 to INTEGER
C**********************************************************************
      INTEGER FUNCTION READ_FLF_SHORT(DATA, SWAP)
      IMPLICIT NONE
C
C     Parameters
C
      INTEGER*2 DATA
      LOGICAL SWAP
C
C     Variables
C
      BYTE      TMPB
      BYTE      B(2)
      INTEGER*2 I2
      EQUIVALENCE (B, I2)
C
      I2 = DATA
      IF (SWAP) THEN
         TMPB = B(1)
         B(1) = B(2)
         B(2) = TMPB
      ENDIF
C
      READ_FLF_SHORT = I2
      RETURN
      END
C
